(***************************************************************************)
(* Program przykladowy pokazujacy sposob wykorzystania driver'a modulu     *)
(* LC-030-1612 firmy AMBEX.                                                *)
(***************************************************************************)
(***************************************************************************)
(* UWAGA: Przed rozpoczeciem kompilacji nalezy sprawdzic ustawienie opcji  *)
(*	kompilatora:                                                       *)
(*	      - alingment = byte,                                          *)
(***************************************************************************)

program test;

uses crt, dos;

{$F+}

{$i ambex-lc.pas}

type
    buftype = array[1..120] of integer;		(* bufor na probki         *)

var
    s_init:       lc0_init;
    s_total:      lc0_total;
    s_module:     lc0_module;
    s_info:       lc0_info;
    s_wait:       lc0_wait;
    s_break:      lc0_break;
    s_data_trans: lc0_transmit;
    s_analog_in:  lc0_analog_in;
    s_leave:      lc0_leave;
    s_interr:     lc0_interrupt;

    r: registers;		(* parametr dla funkcji intr               *)
    LCinterrupt: integer;	(* numer przerwania driver'a               *)
    sstat: integer;		(* slowo komunikacyjne dla INTERRUPT_SERVICE*)
    interruptflag: boolean;	(* flaga ustawiana przez procedure obslugi *)
				(* przerwania mod_interrupt_service        *)

const
    BUFLEN	= 120;				(* dlugosc bufora          *)
    SAMPLES	= 15;				(* liczba probek           *)
    CHANNELS	= 8;				(* liczba kanalow          *)

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Zamiana cyfry szesnastkowej na jej reprezentacje znakowa.             *)
(* Parametry:                                                              *)
(*   dig - cyfra do zamiany                                                *)
(* Wartosc:                                                                *)
(*   Reprezentacja znakowa cyfry.                                          *)
(***************************************************************************)
function hexdigit(dig: word) : char;
begin
    if dig < 10 then hexdigit := chr(ord('0') + dig)
    else             hexdigit := chr(ord('A') + dig - 10);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wydrukowanie liczby calkowitej w postaci szesnastkowej.               *)
(* Parametry:                                                              *)
(*   val - wartosc do wydrukowania                                         *)
(***************************************************************************)
procedure writehex(val: word);
begin
    write(hexdigit((val and $F000) shr 12));
    write(hexdigit((val and $F00) shr 8));
    write(hexdigit((val and $F0) shr 4));
    write(hexdigit(val and $F));
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Inicjalizacja zmiennych.                                              *)
(***************************************************************************)
procedure initprogram;
begin
    s_init.LC0_CODE       := MODULE_INIT;
    s_total.LC0_CODE      := GET_TOTAL_CONFIGURATION;
    s_module.LC0_CODE     := GET_MODULE_CONFIGURATION;
    s_info.LC0_CODE       := GET_INFO;
    s_wait.LC0_CODE       := WAIT_FOR_END;
    s_break.LC0_CODE      := BREAK;
    s_data_trans.LC0_CODE := DATA_TRANSMIT;
    s_analog_in.LC0_CODE  := ANALOG_INPUT;
    s_leave.LC0_CODE      := LEAVE_DRIVER;
    s_interr.LC0_CODE     := INTERRUPT_SERVICE;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Sprawdzenie obecnosci drivera.                                        *)
(* Sposob:                                                                 *)
(*   Przez probe otwarcia urzadzenia o nazwie okreslonej przez driver.     *)
(* Parametry:                                                              *)
(*   name - nazwa urzadzenia                                               *)
(* Wartosc:                                                                *)
(*   TRUE  - driver jest zainstalowany                                     *)
(*   FALSE - driver nie jest zainstalowany                                 *)
(***************************************************************************)
function driverinstalled(name: string) : boolean;
var
    hd: text;
begin
{$I-}
    assign(hd, name);
    reset(hd);
    if IOResult <> 0 then
	driverinstalled := false
    else
	begin
	    close(hd);
	    driverinstalled := true
	end;
{$I+}
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Rozpoznanie, ktory driver (a co za tym idzie ktory modul) jest        *)
(*   zainstalowany.                                                        *)
(* Sposob:                                                                 *)
(*   Przez sprawdzenie zainstalowania kazdego z potencjalnych driver'ow.   *)
(* Uwagi:                                                                  *)
(*   Funkcja dodatkowo wyswietla komunikat o testowanym module i podstawia *)
(*   wlasciwa wartosc na zmienna LCinterrupt (wlasciwy numer przerwania    *)
(*   driver'a). Jezeli zaden driver nie jest zainstalowany to po           *)
(*   wyswietleniu wlasciwego komunikaru program konczy prace.              *)
(***************************************************************************)
procedure checkdrivers;
begin
    clrscr;
    gotoxy(1, 1);
    if driverinstalled('LC3016^^') then
	begin
	    writeln('Test modulu LC-030-1612');
	    LCinterrupt := LC030_16;
	end
    else
	begin
	    writeln('Driver nie jest zainstalowany!');
	    halt;
	end;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Rozpoznanie konfiguracji badanego modulu.                             *)
(* Sposob:                                                                 *)
(*   Przez wykorzystanie funkcji driver'a GET_TOTAL_CONFIGURATION,         *)
(*   GET_MODULE_CONFIGURATION, GET_INFO.                                   *)
(* Uwagi:                                                                  *)
(*   Funkcja wypelnia struktury total, module, info i inicjalizuje modul.  *)
(*    moduly.                                                              *)
(***************************************************************************)
procedure askdriver;
begin
    r.dx := seg(s_total);
    r.di := ofs(s_total);
    intr(LCinterrupt, r);			(* GET_TOTAL_CONFIGURATION *)

				(* inicjalizacja zainstalowanego modulu    *)
    s_init.LC0_IMODULE := s_total.LC0_TONF and $F;
    r.dx := seg(s_init);
    r.di := ofs(s_init);
    intr(LCinterrupt, r);				(* MODULE_INIT     *)

					(* spytanie o konfiguracje modulu  *)
    s_module.LC0_MMODULE := LC0_MODA;
    r.dx := seg(s_module);
    r.di := ofs(s_module);
    intr(LCinterrupt, r);			(* GET_MODULE_CONFIGURATION*)

					(* spytanie o konfiguracje toru a/c*)
    s_info.LC0_GMODULE := LC0_MODA;
    s_info.LC0_GTYPE   := LC0_AINPUT;
    s_info.LC0_GNUM    := 1;
    r.dx := seg(s_info);
    r.di := ofs(s_info);
    intr(LCinterrupt, r);				(* GET_INFO        *)
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wyswietlenie konfiguracji badanego modulu i jego toru a/c.            *)
(* Sposob:                                                                 *)
(*   Przez wykorzystanie informacji zawartych w strukturach module i info. *)
(***************************************************************************)
procedure displayconfiguration;
var
    i: integer;
begin
					(* konfiguracja modulu              *)
    writeln;
    writeln('---------- Konfiguracja modulu:');
    writeln;
    write('Adresy modulu: ');
    writehex(s_module.LC0_MBASE1);
    write(', ');
    writehex(s_module.LC0_MBASE2);
    writeln(' (hex)');
    writeln('Liczba przetwornikow a/c: ', s_module.LC0_MIAD);
    writeln('Liczba przetwornikow c/a: ', s_module.LC0_MIDA);
    writeln('Liczba portow cyfrowych wejsciowych: ', s_module.LC0_MIDI);
    writeln('Liczba portow cyfrowych wyjsciowych: ', s_module.LC0_MIDO);
    writeln('Czestotliwosc zegara modulu:  ',
	    (s_module.LC0_MCLOCK / 1000):4:2, 'MHz');

					(* konfiguracja toru a/c            *)
    writeln;
    writeln('---------- Konfiguracja toru a/c:');
    writeln;
    writeln('Liczba kanalow: ', s_info.LC0_GCHAN);
    writeln('Rozdzielczosc: ', s_info.LC0_GRES, ' bitow');
    writeln('Zakres napiec: ', (s_info.LC0_GMINV / 10):5:1, '..',
	    (s_info.LC0_GMAXV / 10):4:1, ' V');
    if s_info.LC0_GDMA = $FF then
	writeln('Tor a/c nie jest podlaczony do kanalow DMA')
    else
	writeln('Numer kanalu DMA podlaczonego do toru a/c: ',
							s_info.LC0_GDMA);
    writeln('Minimalne okresy probkowania [s]:');
    write('     ');
    for i := 1 to s_info.LC0_GCHAN do
	begin
	    write((s_info.LC0_GMINP[i] / 10):5:1);
	    if i < s_info.LC0_GCHAN then
		begin
		    write(', ');
		    if (i mod 8) = 0 then
			begin
			    writeln;	(* zlamanie linii co 8 wielkosci   *)
			    write('     ');
			end
		end
	end;
    writeln;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Zainstalowanie procedury obslugi przerwania generowanego przez        *)
(*   Ctrl-Break.                                                           *)
(* Sposob:                                                                 *)
(*   Przez wywolanie funkcji BREAK.                                        *)
(***************************************************************************)
procedure installbreak;
begin
    s_break.LC0_BMODE := LC0_BREAK_INST;
    s_break.LC0_BPROC := nil;	(* podlozona zostanie procedura driver'a   *)
    r.dx := seg(s_break);
    r.di := ofs(s_break);
    intr(LCinterrupt, r);				(* BREAK           *)
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Przerwa miedzy kolejnymi czesciami programu - oczekiwanie na reakcje  *)
(*   operatora.                                                            *)
(***************************************************************************)
procedure pressanykey;
var
    c: char;
begin
    writeln;
    writeln('Nacisnij dowolny klawisz . . .');
    c := readkey;
    clrscr;
    gotoxy(1, 1);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wyswietlenie bufora z danymi pomiarowymi.                             *)
(* Parametry:                                                              *)
(*   buf - adres bufora                                                    *)
(*   c   - liczba kanalow                                                  *)
(*   s   - liczba probek na kanal                                          *)
(***************************************************************************)
procedure displaybuf(buf: buftype; c, s: integer);
var
    i: integer;
    j: integer;
begin
    for i := 1 to s do
	begin
	    write(i:2, ': ');
	    for j := 1 to c do
		begin
		    writehex(buf[(i - 1) * c + j]);
		    write('  ');
		end;
	    writeln;
	end;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wypisanie komunikatow o bledzie / ostrzezeniu / dodatkowej informacji *)
(*   bledzie.                                                              *)
(* Parametry:                                                              *)
(*   status   - LC0_STATUS                                                 *)
(*   err_stat - LC0_ERR_STAT                                               *)
(***************************************************************************)
procedure drivererror(status, err_stat: shortint);
begin
    if status > 0 then
	writeln('--------- Ostrzezenie: ', status)
    else
	writeln('--------- Blad: ', status);
    if err_stat < 0 then
	writeln('--------- Informacje dodatkowe: ', err_stat);
end;

(****************************************************************************)
(* Przeznaczenie:                                                           *)
(*   Obsluga przerwania przychodzacego z modulu po zakonczeniu przetwarzania*)
(* Sposob:                                                                  *)
(*   Badanie stanu zmiennej komunikacyjnej sstat. Po zakonczeniu transmisji *)
(*   wyinstalowanie procedury obslugi przerwania i powiadomienie drivera o  *)
(*   zakonczeniu pomiaru.                                                   *)
(* Uwagi:                                                                   *)
(*   Driver przydziela procedurze stos o dlugosci 128 bajtow. Jezeli jest   *)
(*   za malo to procedura powinna przelaczac sie na wlasny stos.            *)
(*   Niezaleznie od tego wymagane jest wylaczenie dla tej procedury badania *)
(*   stosu ($S-).                                                           *)
(****************************************************************************)
{$S-}
procedure mod_interrupt_service;
begin
    if (sstat and LC0_IS_START) <> 0 then
    	writeln('Poczatek -')
    else if (sstat and LC0_IS_END_ADC) <> 0 then
	begin
				    (* wyinstalowanie obslugi przerwania   *)
	    s_interr.LC0_SMODULE := 1;
	    s_interr.LC0_SPROC := nil;
	    r.dx := seg(s_interr);
	    r.di := ofs(s_interr);
	    intr(LCinterrupt, r);		(* INTERRUPT_SERVICE       *)

			    (* powiadomienie driver'a o koncu prztwarzania *)
	    s_wait.LC0_WTYPE := LC0_AINPUT;
	    s_wait.LC0_WMODULE := 1;
	    s_wait.LC0_WNUM := 1;
	    s_wait.LC0_WMODE := LC0_W_FINISHED;
	    r.dx := seg(s_wait);
	    r.di := ofs(s_wait);
	    intr(LCinterrupt, r);			(* WAIT_FOR_END    *)

    	    writeln('Koniec -');
	    interruptflag := true;
	end;
end;
{$S+}

(****************************************************************************)
(* Przeznaczenie:                                                           *)
(*   Odwrocenie kolejnosci kanalow tak, aby kanal 0 byl w kolumnie 0 itd.   *)
(* Parametry:                                                               *)
(*   buf  - bufor danych                                                    *)
(*   len  - liczba probek w buforze                                         *)
(*   chn1 - liczba kanalow podstawowych                                     *)
(*   chn2 - liczba kanalow dodatkowych                                      *)
(*   per2 - krotnosc okresu podstawowego                                    *)
(* Uwagi:                                                                   *)
(*   Poniewaz buf zostal wypelniony przez funkcje driver'a DATA_TRANSMIT to:*)
(*   len = ((per2 - 1) * chn1) + (chn1 + chn2)                              *)
(****************************************************************************)
procedure reverse_data(var buf: buftype; len: longint;
		       chn1, chn2, per2: integer);
var
    lines, lj, ind: longint;
    i, j, n, v: integer;
begin

    if (chn1 = 1) and (chn2 = 0) then
	exit;
    if chn2 = 0 then
	lines := len div chn1		(* praca z jedna czestotliwoscia   *)
    else
	begin				(* praca z dwiema czestotliwosciami*)
	    lj := chn1 * per2 + chn2;
	    lines := (len div lj) * per2;
	    lines := lines + (len mod lj) div chn1;
	end;

    ind := 1;
    for lj := 0 to lines - 1 do
	begin
	    if chn2 = 0 then
		i := chn1
	    else if ((lj + 1) mod per2) = 0 then
	    	i := chn1 + chn2
	    else
	    	i := chn1;
	    n := i div 2;
	    for j := 0 to n - 1 do
		begin
		    v := buf[ind + j];
		    buf[ind + j] := buf[ind + i - j - 1];
		    buf[ind + i - j - 1] := v;
		end;
	    ind := ind + i;
	end;
end;

(****************************************************************************)
(* Przeznaczenie:                                                           *)
(*   Przepisanie do bufora danych z pamieci modulu (LC-030).                *)
(* Sposob:                                                                  *)
(* Parametry:                                                               *)
(*   buf     - bufor danych                                                 *)
(*   buflen  - dlugosc bufora                                               *)
(*   first   - numer pierwszej probki (w kanale podstawowym, liczonej od 1) *)
(*          do przepisania                                                  *)
(*   samples - liczba probek (w kanale podstawowym) do przepisania          *)
(*   chn1    - liczba kanalow podstawowych                                  *)
(*   chn2    - liczba kanalow dodatkowych                                   *)
(*   per2    - krotnosc okresu podstawowego                                 *)
(****************************************************************************)
function transmit_data(var buf: buftype; buflen, first, samples: longint;
		       chn1, chn2, per2: integer) : boolean;
var
    num: longint;	(* bezwzgledny numer pierwszej probki do przepisania *)
    len: longint;	(* bezwzgledna liczba probek do przepisania          *)
label
    drv_error;
begin

    if chn2 = 0 then
	begin
	    num := (first - 1) * chn1;
	    len := samples * chn1;
	end
    else
	begin
	    num := ((first - 1) div per2) * (chn1 * per2 + chn2);
	    len := (samples div per2) * (chn1 * per2 + chn2);
	    len := len + (samples mod per2) * chn1;
	end;

    s_data_trans.LC0_RMODE := LC0_FROM_EXT_DIR;
    s_data_trans.LC0_RADDR := @buf;
    s_data_trans.LC0_RLEN := buflen;
    s_data_trans.LC0_RMEAS := num + 1;
    s_data_trans.LC0_RNUM := len;
    r.dx := seg(s_data_trans);
    r.di := ofs(s_data_trans);
    intr(LCinterrupt, r);				(* DATA_TRANSMIT     *)
    if (s_data_trans.LC0_STATUS <> LC0_OK) and
       (s_data_trans.LC0_STATUS <> LC0_OTHER_LEN) then
	goto drv_error;
    reverse_data(buf, s_data_trans.LC0_RRNUM, chn1, chn2, per2);
    transmit_data := true;
    exit;

drv_error:
    drivererror(s_data_trans.LC0_STATUS, s_data_trans.LC0_ERR_STAT);
    transmit_data := false;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie transmisji blokowej.                                        *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT.                                 *)
(* Parametry:                                                              *)
(*   start - typ warunku startu                                            *)
(*   start    - typ warunku startu                                         */
(*   asynchro - TRUE  - praca asynchroniczna                               *)
(*              FALSE - praca synchroniczna                                *)
(*   interr   - TRUE  - praca z przerwaniami                               *)
(*              FALSE - praca bez przerwan                                 *)
(*              parametr istotny tylko przy pracy asynchronicznej          *)
(* Uwagi:                                                                  *)
(*   Funkcja wywolujaca musi ustawic w strukturze s_analog_in okres        *)
(*   probkowania i parametry warunku startu.                               *)
(***************************************************************************)
procedure transmission(start: byte; asynchro, interr: boolean);
var
    buf: buftype;			(* bufor na probki                 *)
    c: char;
label
    ok, drv_error, display;
begin
    write('Pomiar blokowy, ', CHANNELS, ' kanalow, ', SAMPLES, ' probek, ');
    if s_analog_in.LC0_APER < 10000 then
    	writeln('okres probkowania ',
	    	(s_analog_in.LC0_APER / 10):5:1, ' s:')
    else if s_analog_in.LC0_APER < 10000000 then
    	writeln('okres probkowania ',
	    	(s_analog_in.LC0_APER / 10000):5:1, ' ms:')
    else
    	writeln('okres probkowania ',
	    	(s_analog_in.LC0_APER / 10000000):5:1, ' s:');
    case start of
	LC0_SIMMED:
	    writeln('Warunek startu: natychmiast');
	LC0_STIME:
	    begin
	        writeln('Warunek startu: uplyw ', s_analog_in.LC0_ASTART.time,
		        ' sekund');
		writeln('Oczekiwanie -');
	    end;
				(* pozostale warunki nie sa obslugiwane bo *)
				(* w programie nie sa wykorzystywane       *)
	end;
    s_analog_in.LC0_AMODULE := LC0_MODA;
    s_analog_in.LC0_ANUM := 1;
    s_analog_in.LC0_AMODE :=
		    LC0_MOD_START or 		(* zezwolenie na pomiar    *)
		    LC0_MOD_NEW_PAR or		(* nowe parametry          *)
		    LC0_MOD_BLOCK;		(* praca blokowa           *)
    if not asynchro then
	begin
	    s_analog_in.LC0_AMODE := s_analog_in.LC0_AMODE +
	    			     LC0_MOD_SYNCHR; (* praca synchroniczna *)
	    interr := false;
	end;
    if interr then
				(* praca z przerwaniami *)
	s_analog_in.LC0_AMODE := s_analog_in.LC0_AMODE + LC0_MOD_INTR; 

				(* stop po zmierzeniu okreslonej liczby    *)
				(* probek                                  *)
    s_analog_in.LC0_ASTST := start + LC0_ZSAMPLES;
				(* praca wielokanalowa, CHANNELS kanalow   *)
    s_analog_in.LC0_ACHAN := CHANNELS;
    s_analog_in.LC0_ACHAN2 := 0;	(* probkowanie z 1 czestotliwoscia *)
    s_analog_in.LC0_AADDR := @buf;
    s_analog_in.LC0_ALEN := BUFLEN;	(* dlugosc bufora                  *)
    s_analog_in.LC0_ABMAR := 0;
    s_analog_in.LC0_AEMAR := 0;		(* oba marginesy zerowe            *)
					(* calkowita liczba probek         *)
    s_analog_in.LC0_ASTOP.samples := SAMPLES * CHANNELS;
    if asynchro and interr then	(* przygotowanie pracy z przerwaniami      *)
	begin
	    s_interr.LC0_SMODULE := 1;
	    s_interr.LC0_SPROC := @mod_interrupt_service;
	    sstat := 0;
	    s_interr.LC0_SSTAT := @sstat;
	    r.dx := seg(s_interr);
	    r.di := ofs(s_interr);
				(* spowodowanie ze driver (a co za tym     *)
				(* idzie procedura mod_interrupt_service)  *)
				(* dostanie wlasciwy segment DS; moze tu   *)
				(* byc uzyta dowolna zmienna byle globalna *)
				(* - umieszczona w segmencie DS            *)
	    r.ds := seg(sstat);
	    intr(LCinterrupt, r);		(* INTERRUPT_SERVICE       *)
	    if s_interr.LC0_STATUS <> LC0_OK then
		begin
		    drivererror(s_interr.LC0_STATUS, s_interr.LC0_ERR_STAT);
		    pressanykey;
		    exit;
		end;
	    interruptflag := false;
	end;
    r.dx := seg(s_analog_in);
    r.di := ofs(s_analog_in);
    intr(LCinterrupt, r);			(* ANALOG_INPUT           *)
    if s_analog_in.LC0_STATUS <> LC0_OK then
	goto drv_error;
ok:
    if not asynchro then			(* praca synchroniczna     *)
	goto display
    else if interr then		(* praca asynchroniczna z przerwaniami     *)
	begin
	    repeat
		;
	    until ((sstat and LC0_IS_BROKEN) <> 0) or interruptflag;
	    if (sstat and LC0_IS_BROKEN) <> 0 then
		exit
	    else
		goto display;
	end
    else			(* praca asynchroniczna bez przerwan       *)
	begin
	    s_wait.LC0_WTYPE := LC0_AINPUT;
	    s_wait.LC0_WMODULE := 1;
	    s_wait.LC0_WNUM := 1;
	    s_wait.LC0_WMODE := LC0_W_TEST;
	    r.dx := seg(s_wait);
	    r.di := ofs(s_wait);
						(* czekanie na koniec      *)
	    repeat
		intr(LCinterrupt, r);			(* WAIT_FOR_END    *)
	    until s_wait.LC0_STATUS <> LC0_IN_PROGRESS;
	    if s_wait.LC0_STATUS = LC0_OK then		(* koniec poprawny *)
		goto display
	    else				(* koniec Ctrl-Break       *)
		begin
		    drivererror(s_wait.LC0_STATUS, s_wait.LC0_ERR_STAT);
		    pressanykey;
		end;
	end;
    exit;

drv_error:
    if s_analog_in.LC0_STATUS = LC0_OTHER_LEN then
	goto ok
    else
	begin
	    drivererror(s_analog_in.LC0_STATUS, s_analog_in.LC0_ERR_STAT);
	    pressanykey;
	end;
    exit;

display:
    if transmit_data(buf, BUFLEN, 1, SAMPLES, CHANNELS, 0, 1) then
	displaybuf(buf, CHANNELS, SAMPLES);
    pressanykey;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie poprawnej transmisji blokowej i wyswietlenie zmierzonych    *)
(*   wartosci.                                                             *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji transmission.                                 *)
(***************************************************************************)
procedure blocktransmission;
begin
    writeln('---------- Poprawne przetwarzanie blokowe');
    writeln;
    s_analog_in.LC0_APER := s_info.LC0_GMINP[CHANNELS];
    transmission(LC0_SIMMED, false, false);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie blednej transmisji blokowej.                                *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT z okresem probkowania mniejszym  *)
(*   niz minimalny wskazany przez driver.                                  *)
(***************************************************************************)
procedure failblocktransmission;
begin
    writeln('---------- Bledne przetwarzanie blokowe');
    writeln;
    s_analog_in.LC0_APER := s_info.LC0_GMINP[CHANNELS] - 1;
    transmission(LC0_SIMMED, false, false);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie poprawnej transmisji blokowej ale przerwanej przez operatora*)
(*   (Ctrl-Break) w trakcie czekania na spelnienie warunku startu.         *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT z warunkiem startu LC0_STIME     *)
(*   (start po okreslonym czasie) i parametrem tego warunku - 1000s.       *)
(***************************************************************************)
procedure interruptedbefore;
begin
    writeln('---------- Przetwarzanie blokowe z oczekiwaniem 1000s');
    writeln('---------- Oczekiwanie nalezy przerwac Ctrl-Break');
    writeln;
    s_analog_in.LC0_APER := s_info.LC0_GMINP[CHANNELS];
    s_analog_in.LC0_ASTART.time := 1000;
    transmission(LC0_STIME, false, false);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie poprawnej transmisji blokowej ale przerwanej przez operatora*)
(*   (Ctrl-Break) w trakcie pomiaru.                                       *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT z okresem probkowania 100s.      *)
(***************************************************************************)
procedure interruptedafter;
begin
    writeln('---------- Przetwarzanie blokowe z okresem probkowania 100s');
    writeln('---------- Pomiar nalezy przerwac Ctrl-Break');
    writeln;
    s_analog_in.LC0_APER := 1000000000;
    transmission(LC0_SIMMED, false, false);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*    Wykonanie transmisji blokowej asynchronicznej bez przerwan.          *)
(***************************************************************************)
procedure asynchronointerr;
begin
    writeln('---------- Przetwarzanie blokowe asynchroniczne bez przerwan');
    writeln;
    s_analog_in.LC0_APER := 1000000;
    transmission(LC0_SIMMED, true, false);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*    Wykonanie transmisji blokowej asynchronicznej z przerwaniami.        *)
(***************************************************************************)
procedure asynchrointerr;
begin
    writeln('---------- Przetwarzanie blokowe asynchroniczne z przerwaniami');
    writeln;
    s_analog_in.LC0_APER := 1000000;
    s_analog_in.LC0_ASTART.time := 2;
    transmission(LC0_STIME, true, true);
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Wykonanie pomiaru bloku probek za pomoca tranmsisji pojedynczej, przy *)
(*   czym caly blok ma byc zmierzony po 10s od startu.                     *)
(* Sposob:                                                                 *)
(*   Przez wykonanie funkcji ANALOG_INPUT.                                 *)
(* Parametry:                                                              *)
(* Wartosc:                                                                *)
(* Uwagi:                                                                  *)
(***************************************************************************)
procedure singletransmission;
var
    buf: buftype;			(* bufor na probki                 *)
    i: integer;
    j: integer;
begin
    writeln('---------- Przetwarzanie pojedyncze - po 5 sekundach');
    writeln;
    s_analog_in.LC0_AMODULE := LC0_MODA;
    s_analog_in.LC0_ANUM := 1;
    s_analog_in.LC0_AMODE := LC0_MOD_START or	(* zezwolenie na pomiar    *)
			     LC0_MOD_NEW_PAR or	(* nowe parametry          *)
			     LC0_MOD_SINGLE;	(* praca pojedyncza        *)
				(* praca wielokanalowa, CHANNELS kanalow   *)
    s_analog_in.LC0_ACHAN := CHANNELS;
    s_analog_in.LC0_AADDR := @buf;
    s_analog_in.LC0_ALEN := CHANNELS;	(* dlugosc bufora (tylko na jeden  *)
    					(* pomiar)                         *)
    s_analog_in.LC0_ASTST := LC0_STIME;
    s_analog_in.LC0_ASTART.time := 5;	(* start po 5 sekundach            *)
    r.dx := seg(s_analog_in);
    r.di := ofs(s_analog_in);
    intr(LCinterrupt, r);				(* ANALOG_INPUT    *)
    s_analog_in.LC0_ASTST := LC0_SIMMED;	(* nastepne probki maja byc*)
						(* pobierane z maksymalna  *)
						(* szybkoscia              *)
    s_analog_in.LC0_AADDR := @buf[CHANNELS + 1];
    r.dx := seg(s_analog_in);
    r.di := ofs(s_analog_in);
    intr(LCinterrupt, r);				(* ANALOG_INPUT    *)
					(* zgaszenie bitu LC0_MOD_NEW_PAR  *)
					(* (zeby driver niepotrzebnie nie  *)
					(* analizowal parametrow)          *)
    s_analog_in.LC0_AMODE := s_analog_in.LC0_AMODE and (not LC0_MOD_NEW_PAR);
    for i := 3 to SAMPLES do
	begin
	    s_analog_in.LC0_AADDR := @buf[(i - 1) * CHANNELS + 1];
	    r.dx := seg(s_analog_in);
	    r.di := ofs(s_analog_in);
	    intr(LCinterrupt, r);			(* ANALOG_INPUT    *)
	end;
    displaybuf(buf, CHANNELS, SAMPLES);
    pressanykey;
end;

(***************************************************************************)
(* Przeznaczenie:                                                          *)
(*   Zakonczenie pracy programu.                                           *)
(* Sposob:                                                                 *)
(*   Rozstanie sie z driver'em za pomoca funkcji LEAVE_DRIVER.             *)
(***************************************************************************)
procedure quit;
begin
    r.dx := seg(s_leave);
    r.di := ofs(s_leave);
    intr(LCinterrupt, r);				(* LEAVE_DRIVER    *)

    writeln('Dziekuje, to wszystko!');
end;


begin
	initprogram;		(* inicjalizacja programu                  *)
	checkdrivers;		(* rozpoznanie zainstalowanego driver'a    *)
	askdriver;		(* odpytanie driver'a o konfiguracje       *)
	displayconfiguration;	(* wyswietlenie konfiguracji modulu i      *)
				(* toru a/c                                *)
	installbreak;		(* zainstalowanie procedury obslugi        *)
				(* przerwania generowanego przez Ctrl-Break*)
	pressanykey;		(* czekanie na operatora                   *)
	blocktransmission;	(* wykonanie transmisji blokowej +         *)
				(* wyswietlenie zmierzonych wartosci       *)
	failblocktransmission;	(* wykonanie blednej transmisji blokowej   *)
	interruptedbefore;	(* wykonanie transmisji blokowej przerwanej*)
				(* przez operatora przed startem           *)
	interruptedafter;	(* wykonanie transmisji blokowej przerwanej*)
				(* przez operatora po starcie              *)
	asynchronointerr;	(* wykonanie transmisji blokowej           *)
				(* asynchronicznej bez przerwan            *)
	asynchrointerr;		(* wykonanie transmisji blokowej           *)
				(* asynchronicznej z przerwaniem           *)
	{
	singletransmission;	(* wykonanie pomiaru w trybie pojedynczym  *)
	}
	quit;			(* zakonczenie programu                    *)
end.
